﻿// --------------------------------------------------------------------------------------
//
// HochRegal1                                                      Ulrich Müller 07.04.11
//                                                                 Aspire C# 2005 Express
// ROBO TX Automation Robots : Modell Hochregal1 (1:1 Umsetzung des RoboPro Programms
// M1 / C1 / I1 : Turm horizontal (Encodermotor)
// M2 / I2 (links : hinten) / I4 (vorn) : Greifer horizontal
// M3 / C3 / I3 : Arm vertikal (Encodermotor)
// I5 / I6 / : Ablegen / Holen (Taster)
//
// --------------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using FishFaceTX;

namespace HochRegal1 {

  public partial class frmMain : Form {
    FishFace tx = new FishFace();
    int posTurm = 0;
    int posArm = 0;
    const Mot mTurm = Mot.M1;
    const Unv iTurm = Unv.I1;
    const Cnt cTurm = Cnt.C1;
    const Mot mGreifer = Mot.M2;
    const Unv iGreiferH = Unv.I2;
    const Unv iGreiferV = Unv.I4;
    const Mot mArm = Mot.M3;
    const Unv iArm = Unv.I3;
    const Cnt cArm = Cnt.C3;

    private void Action() {
      Referenzfahrt();
      tx.Pause(1000);
      Bereitstellung();
      GabelVor();
      GabelHeben();
      GabelZurueck();
      Platz1();
      GabelHeben();
      GabelVor();
      GabelSenken();
      GabelZurueck();
      Referenzfahrt();
    }

    private void Platz1() {
      tx.StartRobMotor(mArm, Dir.Right, 512, 1720);
      tx.StartRobMotor(mTurm, Dir.Right, 512, 3070);
      tx.WaitForMotors(mArm, mTurm);
      posArm += 1720;
      posTurm += 3070;
    }


    private void Bereitstellung() {
      tx.StartRobMotor(mArm, Dir.Right, 512, 40);
      tx.WaitForMotors(mArm);
      posArm += 40;
    }

    private void Referenzfahrt() {
      GabelZurueck();
      tx.StartRobMotor(mTurm, Dir.Left, 512, 9999);
      tx.StartRobMotor(mArm, Dir.Left, 512, 9999);
      tx.WaitForMotors(mTurm, mArm);
      posArm = 0;
      posTurm = 0;
    }

    private void GabelHeben() {
      tx.StartRobMotor(mArm, Dir.Right, 512, 150);
      tx.WaitForMotors(mArm);
      posArm += 150;
    }

    private void GabelSenken() {
      tx.StartRobMotor(mArm, Dir.Left, 512, 150);
      tx.WaitForMotors(mArm);
      posArm -= 150;
    }

    private void GabelVor() {
      if (tx.GetInput(iGreiferV)) return;
      tx.SetMotor(mGreifer, Dir.Right);
      tx.WaitForHigh(iGreiferV);
      tx.SetMotor(mGreifer, Dir.Off);
    }

    private void GabelZurueck() {
      if(tx.GetInput(iGreiferH)) return;
      tx.SetMotor(mGreifer, Dir.Left);
      tx.WaitForHigh(iGreiferH);
      tx.SetMotor(mGreifer, Dir.Off);
    }

    #region --- Programm-Kontrolle ---
    public frmMain() {
      InitializeComponent();
    }

    private void cmdAction_Click(object sender, EventArgs e) {
      try {
        tx.OpenController(txtCom.Text);
        cmdAction.Enabled = false;
        cmdEnde.Text = "&HALT";
        lblStatus.Text = "--- Bei der Arbeit ---";
        lblHinweis.Text = "--- Ende : HALT-Button oder ESC-Taste ---";

        Action();        // --- Haupt-Routine der Anwendung

      }
      catch (FishFaceException txe) {
        MessageBox.Show(txe.Message, this.Text,
          MessageBoxButtons.OK, MessageBoxIcon.Stop);
      }
      finally {
        tx.CloseController();
        cmdAction.Enabled = true;
        cmdEnde.Text = "&ENDE";
        lblStatus.Text = "--- Auf ein Neues ---";
        lblHinweis.Text = "Nochmal : ACTION, sonst ENDE oder ESC-Taste";
        cmdAction.Focus();
      }
    }

    private void cmdEnde_Click(object sender, EventArgs e) {
      // --- Programmende nur, wenn Action nicht läuft ---
      if (cmdEnde.Text == "&HALT") tx.NotHalt = true; else this.Close();
    }

    private void formClosing(object sender, FormClosingEventArgs e) {
      // --- Wenn Action läuft, wird ein Programmabbruch über (x) unterbunden ------
      if (cmdEnde.Text == "&HALT") e.Cancel = true;
    }
    #endregion
  }
}