﻿// --------------------------------------------------------------------------------------
//
// Hochregal3c                                                     Ulrich Müller 09.04.11
//                                                                 Aspire C# 2005 Express
// ROBO TX Automation Robots : Modell Hochregallager 2
// Speichern und Ausgeben von Lagergut. Nach RoboPro Hochregal2 und 3
// Erstes freies / belegtes Fach
// 
// --------------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using FishFaceTX;

namespace Hochregal3c {
  public partial class frmMain : Form {
    FishControl fc = new FishControl();
    RobMotors ArmTurm;        // --- Kombination Turm und Arm mit Endtastern
    LimitedMotor Gabel;       // --- Gabel mit Motor und Endtastern
    PushButton Fetch;         // --- Fetch Taster
    PushButton Store;         // --- Store Taster
    int[,] PosListe;          // --- Liste der Lagerplätze x, y ---
    bool[] BelegtListe;       // --- Liste mit dem Status der Fächer

    private void Action() {
      Gabel.MoveLeft();            // --- nach Hause ---
      ArmTurm.MoveHome();
      do {
        lblStatus.Text = "I5 : Ablegen, I6 Holen";
        if (Store.IsTrue) {
          // --- Store ---
          lblStatus.Text = "Legt die Tonne ab";
          Ablegen();
          ArmTurm.MoveHome();         // --- nach Hause ---
        }
        else if (Fetch.IsTrue) {
          // --- Fetch ---
          lblStatus.Text = "Holt die Tonne";
          Holen();
          ArmTurm.MoveHome();         // --- nach Hause ---
        }
        fc.Pause(333);
      } while (!fc.Finish());
    }

    private void Holen() {
      // --- Suchen ersten belegten Platz ---
      int holPos = -1;
      for (int i = 0; i < BelegtListe.Length - 1; i++) {
        if (BelegtListe[i]) {
          holPos = i;
          break;
        }
      }
      if (holPos < 0) {
        lblStatus.Text = "Lager ist leer";
        return;
      }
      
      // --- Lagerplatz anfahren ---
      ArmTurm.MoveTo(PosListe[holPos,0], PosListe[holPos,1]);
      // --- Gabel vor, heben, zurück ---
      Gabel.MoveRight();
      ArmTurm.MoveDelta(0, 150);
      Gabel.MoveLeft();
      // --- Ablagetisch anfahren ---
      ArmTurm.MoveTo(0, 220);
      // --- Gabel vor, senken, zurück ---
      Gabel.MoveRight();
      ArmTurm.MoveDelta(0, -150);
      Gabel.MoveLeft();
      BelegtListe[holPos] = false;
    }

    private void Ablegen() {
      // --- Suchen ersten freien Platz ---
      int zielPos = - 1;
      for (int i = 0; i < BelegtListe.Length - 1; i++) {
        if (!BelegtListe[i]) {
          zielPos = i;
          break;
        }
      }
      if (zielPos < 0) {
        lblStatus.Text = "Lager ist voll belegt";
        return;
      }
      
      // --- Ablagetisch anfahren ---
      ArmTurm.MoveTo(0, 40);
      // --- Gabel vor, heben, zurück ---
      Gabel.MoveRight();
      ArmTurm.MoveDelta(0, 150);
      Gabel.MoveLeft();
      // --- Lagerplatz anfahren ---
      ArmTurm.MoveTo(PosListe[zielPos, 0], PosListe[zielPos, 1]);
      // --- Gabel heben, vor, senken, zurück ---
      ArmTurm.MoveDelta(0, 150);
      Gabel.MoveRight();
      ArmTurm.MoveDelta(0, -150);
      Gabel.MoveLeft();
      BelegtListe[zielPos] = true;
    }

    #region --- Programm-Kontrolle ---
    public frmMain() {
      InitializeComponent();
      ArmTurm = new RobMotors(fc, Ctr.Main,
        new RobData(Mot.M1, 5200), new RobData(Mot.M3, 2500));
      Gabel = new LimitedMotor(fc, Ctr.Main, Mot.M2, Unv.I2, Unv.I4);
      Store = new PushButton(fc, Ctr.Main, Unv.I5);
      Fetch = new PushButton(fc, Ctr.Main, Unv.I6);
      PosListe = new int[,] { { 3100,   40 }, { 5000,   40 }, 
                              { 3100,  990 }, { 5000,  990 }, 
                              { 3100, 1900 }, { 5000, 1900 } };
      BelegtListe = new bool[] { false, false, false, false, false, false };
    }

    private void cmdAction_Click(object sender, EventArgs e) {
      try {
        fc.Connect(txtCom.Text);
        cmdAction.Enabled = false;
        cmdEnde.Text = "&HALT";
        lblStatus.Text = "--- Bei der Arbeit ---";
        lblHinweis.Text = "--- Ende : HALT-Button oder ESC-Taste ---";

        Action();        // --- Haupt-Routine der Anwendung

      }
      catch (FishFaceException txe) {
        MessageBox.Show(txe.Message, this.Text,
          MessageBoxButtons.OK, MessageBoxIcon.Stop);
      }
      finally {
        fc.DisConnect();
        cmdAction.Enabled = true;
        cmdEnde.Text = "&ENDE";
        lblStatus.Text = "--- Auf ein Neues ---";
        lblHinweis.Text = "Nochmal : ACTION, sonst ENDE oder ESC-Taste";
        cmdAction.Focus();
      }
    }

    private void cmdEnde_Click(object sender, EventArgs e) {
      // --- Programmende nur, wenn Action nicht läuft ---
      if (cmdEnde.Text == "&HALT") fc.NotHalt = true; else this.Close();
    }

    private void formClosing(object sender, FormClosingEventArgs e) {
      // --- Wenn Action läuft, wird ein Programmabbruch über (x) unterbunden ------
      if (cmdEnde.Text == "&HALT") e.Cancel = true;
    }
    #endregion
  }
}